"use strict";
/*----------------------------------
- DEPENDENCIES
----------------------------------*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
// Npm
var isURL_1 = __importDefault(require("validator/lib/isURL"));
// Interval
var types_1 = require("./types");
var BadRequest = /** @class */ (function (_super) {
    __extends(BadRequest, _super);
    function BadRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return BadRequest;
}(Error));
/*----------------------------------
- METHODS
----------------------------------*/
exports["default"] = (function (input) {
    // TODO: Check remaining requests / minute
    // Method
    if (input.method !== undefined) {
        if (!types_1.allowedMethods.includes(input.method))
            throw new BadRequest("Only the following HTTP methods are currently allowed: " + types_1.allowedMethods.join(', '));
    }
    else
        input.method = 'GET';
    // URL
    if (typeof input.url !== "string" || !(0, isURL_1["default"])(input.url, {
        require_protocol: true,
        require_valid_protocol: true,
        protocols: ['http', 'https'],
        require_host: true,
        require_port: false,
        allow_protocol_relative_urls: false
    }))
        throw new BadRequest("The url parameter must be a valid URL string: protocol (http or https) + domain + path (optional)");
    // Cookies
    if (input.cookies !== undefined) {
        // Type
        if (typeof input.cookies !== 'string')
            throw new BadRequest("The cookie parameter must be a string. Example: user=Bob; age=28;");
    }
    // body
    if (input.body !== undefined) {
        // Bodytype
        if (input.bodyType === undefined)
            throw new BadRequest("The bodyType parameter must be provided when the body parameter is specified.");
        if (!types_1.bodyTypes.includes(input.bodyType))
            throw new BadRequest("Invalid value for the bodyType parameter. Allowed values: " + types_1.bodyTypes.join(', '));
        // Type
        if (typeof input.body !== 'object' || input.body.constructor.name !== 'Object')
            throw new BadRequest("The body parameter must be an object.");
    }
    if (input.extract !== undefined)
        validateExtractors(input.extract, 'extract');
    if (input.withBody !== undefined && typeof input.withBody !== "boolean")
        throw new BadRequest("The withBody parameter must be a boolean.");
    if (input.withHeaders !== undefined && typeof input.withHeaders !== "boolean")
        throw new BadRequest("The withHeaders parameter must be a boolean.");
    return input;
});
var validateExtractors = function (extract, path) {
    if (!extract || typeof extract !== 'object')
        throw new BadRequest("The " + path + " option must be an object or an array (" + typeof extract + " given).");
    if (Array.isArray(extract)) {
        /*
            extract: ["h4", "text", true, "title"]
        */
        if (extract.length < 3 || typeof extract[0] !== "string" || typeof extract[1] !== "string" || typeof extract[2] !== "boolean")
            throw new BadRequest("When the " + path + " option is an array, it must contain at least 3 values: "
                + "CSS selector (string), attribute (string), required (boolean) and optionnaly filters (strings)");
    }
    else {
        /*
            extract: {
                "$foreach": ".sh-dgr__content",
                "name": ["h4", "text", true, "title"],
                ...
            }
        */
        if (Object.keys(extract).length === 0)
            throw new BadRequest("The " + path + " parameter must contain at least one entry.");
        // @ts-ignore: Property '$foreach' does not exist on type 'TExtractor'
        var _a = extract, $foreach = _a.$foreach, toExtract = __rest(_a, ["$foreach"]);
        // Foreach
        if ($foreach !== undefined && typeof $foreach !== "string")
            throw new BadRequest("When specified, the " + path + ".$foreach parameter must be a CSS selector string.");
        // For each extractor
        for (var name_1 in toExtract) {
            validateExtractors(toExtract[name_1], path + '.' + name_1);
        }
    }
    return extract;
};
